<?php

namespace Emulator\Network\Game;

use Emulator\Core\Logging\Logger;
use Emulator\Encoding\ByteEncoding;
use Emulator\Messages\Incoming\ClientMessage;
use Emulator\Messages\MessageHandler;
use Emulator\Network\Game\Sessions\Session;
use Override;
use Ratchet\ConnectionInterface;
use Ratchet\Http\HttpServer;
use Ratchet\RFC6455\Messaging\MessageInterface;
use Ratchet\Server\IoServer;
use Ratchet\WebSocket\MessageComponentInterface;
use Ratchet\WebSocket\WsConnection;
use Ratchet\WebSocket\WsServer;
use React\EventLoop\Loop;
use React\Socket\SocketServer;

class GameNetworkServer implements MessageComponentInterface {
    private static ?GameNetworkServer $instance = null;
    private array $sessions;

    public function start(int $port): void {
        $loop = Loop::get();

        $this->sessions = [];
        $socketServer = new SocketServer("127.0.0.1:{$port}", loop: $loop);
        new IoServer(
            new HttpServer(
                new WsServer($this)
            ),
            $socketServer,
            $loop
        );
    }

    public static function getInstance(): GameNetworkServer {
        if (self::$instance === null) {
            self::$instance = new self();
        }

        return self::$instance;
    }

    #[Override] function onOpen(ConnectionInterface $conn): void {
        $this->sessions[$conn->resourceId] = new Session(new WsConnection($conn));
    }

    #[Override] function onClose(ConnectionInterface $conn): void {
        unset($this->sessions[$conn->resourceId]);
    }

    #[Override] function onError(ConnectionInterface $conn, \Exception $e): void {
        Logger::fatal($e->getMessage());
    }

    #[Override] public function onMessage(ConnectionInterface $conn, MessageInterface $msg): void {
        $raw = $msg->getPayload();
        $data = unpack('C*', $raw);

        if (count($data) >= 6) {
            $length = ByteEncoding::getInt32(array_slice($data, 1, 4));
            $packet = array_slice($data, 4, $length);

            MessageHandler::getInstance()->handlePacket($this->sessions[$conn->resourceId], new ClientMessage($packet));
        }
    }
}