<?php

namespace Emulator\Network\Game\Sessions;

use Emulator\Encoding\ByteEncoding;
use Emulator\Messages\Outgoing\IMessageComposer;
use Emulator\Messages\Outgoing\ServerMessage;
use Ratchet\RFC6455\Messaging\Frame;
use Ratchet\RFC6455\Messaging\Message;
use Ratchet\WebSocket\WsConnection;

readonly class Session {
    private WsConnection $connection;

    public function __construct(WsConnection $connection) {
        $this->connection = $connection;
    }

    public function sendComposer(IMessageComposer $composer): void {
        $message = new ServerMessage($composer->getHeader()->value);
        $composer->compose($message);
        $data = array_merge(ByteEncoding::int32ToArray(count($message->getPacket())), $message->getPacket());

        $this->connection->send(new Frame(pack('C*', ...$data), true, Frame::OP_BINARY));
    }

    function encode($text) {
        $b1 = 0x80 | (0x1 & 0x0f);
        $length = strlen($text);
        if ($length <= 125)
            $header = pack('CC', $b1, $length);
        elseif($length > 125 && $length < 65536)
            $header = pack('CCn', $b1, 126, $length);
        elseif($length >= 65536)
            $header = pack('CCNN', $b1, 127, $length);

        return $header.$text;
    }

}